# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'CurrencyRate.rate'
        db.alter_column(u'metasettings_currencyrate', 'rate', self.gf('django.db.models.fields.DecimalField')(max_digits=7, decimal_places=2))

    def backwards(self, orm):

        # Changing field 'CurrencyRate.rate'
        db.alter_column(u'metasettings_currencyrate', 'rate', self.gf('django.db.models.fields.DecimalField')(max_digits=5, decimal_places=2))

    models = {
        u'metasettings.currencyrate': {
            'Meta': {'object_name': 'CurrencyRate'},
            'currency': ('django.db.models.fields.CharField', [], {'max_length': '3'}),
            'date_last_sync': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'month': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rate': ('django.db.models.fields.DecimalField', [], {'max_digits': '7', 'decimal_places': '2'}),
            'year': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['metasettings']


from django.db import models

from metasettings.fields import CurrencyField


class Project(models.Model):
    name = models.CharField(max_length=100)
    currency = CurrencyField()


class AllowNull(models.Model):
    currency = CurrencyField(null=True)



from django.conf.urls import patterns, url

from .views import dashboard


urlpatterns = patterns(
    '',
    url(r'^dashboard/(?:(?P<status>(language|currency)+)/)?$',
        dashboard,
        name='metasettings_dashboard'),
)



from django.conf import settings

from . import choices


CURRENCY_COOKIE_NAME = getattr(settings, 'METASETTINGS_CURRENCY_COOKIE_NAME', 'django_currency')

DEFAULT_CURRENCY = getattr(settings, 'METASETTINGS_DEFAULT_CURRENCY', 'EUR')

CURRENCY_CHOICES = getattr(settings, 'METASETTINGS_CURRENCY_CHOICES', choices.CURRENCY_CHOICES)

CURRENCY_TRIGRAMS = getattr(settings, 'METASETTINGS_CURRENCY_TRIGRAMS', choices.CURRENCY_TRIGRAMS)

CURRENCY_LABELS = getattr(settings, 'METASETTINGS_CURRENCY_LABELS', choices.CURRENCY_LABELS)

CURRENCY_BY_COUNTRIES = getattr(settings, 'METASETTINGS_CURRENCY_BY_COUNTRIES', choices.CURRENCY_BY_COUNTRIES)

CURRENCY_SYMBOLS = getattr(settings, 'METASETTINGS_CURRENCY_SYMBOLS', choices.CURRENCY_SYMBOLS)



import json
import requests
import logging

from .models import CurrencyRate

LOGGER = logging.getLogger(__name__)


def rate_request(app_id, date=None):
    url = 'http://openexchangerates.org/api'
    if date:
        url += '/historical/%s.json' % date.strftime("%Y-%m-%d")
    else:
        url += '/latest.json'

    response = requests.get(url, params={'app_id': app_id})

    if response.status_code == 200:
        return json.loads(response.content)

    LOGGER.warning("Request to %s returned %s", url, response.status_code)


def sync_rates(app_id, date=None):
    result = rate_request(app_id, date=None)

    if not result:
        return

    for currency, rate in result['rates'].iteritems():
        currency_rate, created = CurrencyRate.objects.update_or_create(
            currency, rate, date
        )
        if currency_rate:
            if created:
                msg = 'Create currency %s with %s'
            else:
                msg = 'Syncing currency %s with %s'

            LOGGER.info(msg, currency, rate)



import re

from django import template
from django.template import TemplateSyntaxError

from metasettings.models import (get_currency_from_request as get_currency,
                                 get_language_from_request as get_language,
                                 convert_amount as _convert_amount)


register = template.Library()

kwarg_re = re.compile(r"(?:(\w+)=)?(.+)")


class Node(template.Node):
    def __init__(self, request, func, var_name):
        self.request = template.Variable(request)
        self.var_name = var_name
        self.func = func

    def render(self, context):
        try:
            request = self.request.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        result = self.func(request)

        if self.var_name:
            context[self.var_name] = result
            return ''

        return result


@register.tag
def get_currency_from_request(parser, token):
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError("%s tag requires arguments" % token.contents.split()[0])

    m = re.search(r'(.*?) as (\w+)', arg)

    if not m:
        raise template.TemplateSyntaxError("%s tag had invalid arguments" % tag_name)

    request, var_name = m.groups()

    return Node(request, get_currency, var_name)


@register.tag
def get_language_from_request(parser, token):
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        raise template.TemplateSyntaxError("%s tag requires arguments" % token.contents.split()[0])

    m = re.search(r'(.*?) as (\w+)', arg)

    if not m:
        raise template.TemplateSyntaxError("%s tag had invalid arguments" % tag_name)

    request, var_name = m.groups()

    return Node(request, get_language, var_name)


class ConvertAmountNode(template.Node):
    @classmethod
    def parse_params(cls, parser, bits):
        args, kwargs = [], {}
        for bit in bits:
            name, value = kwarg_re.match(bit).groups()
            if name:
                kwargs[name] = parser.compile_filter(value)
            else:
                args.append(parser.compile_filter(value))
        return args, kwargs

    @classmethod
    def handle_token(cls, parser, token):
        bits = token.split_contents()
        name = bits[0]

        if len(bits) < 4:
            raise TemplateSyntaxError("'%s' takes at least 3 argument" % name)

        asvar = None

        if 'as' in bits:
            pivot = bits.index('as')

            try:
                asvar = bits[pivot + 1]
            except IndexError:
                raise TemplateSyntaxError("'%s' arguments must include "
                                          "a variable name after 'as'" % name)
            del bits[pivot:pivot + 2]

        cls_args, cls_kwargs = cls.parse_params(parser, bits[1:])

        cls_kwargs['asvar'] = asvar

        return cls(*cls_args, **cls_kwargs)

    def __init__(self, from_currency, to_currency, amount, ceil=False, asvar=None):
        self.from_currency = from_currency
        self.to_currency = to_currency
        self.amount = amount
        self.ceil = ceil
        self.asvar = asvar

    def render(self, context):
        from_currency = self.from_currency.resolve(context)

        to_currency = self.to_currency.resolve(context)

        amount = self.amount.resolve(context)

        ceil = self.ceil and self.ceil.resolve(context) or self.ceil

        amount = _convert_amount(from_currency,
                                 to_currency,
                                 amount,
                                 ceil=ceil)

        if self.asvar:
            context[self.asvar] = amount
            return ''

        return amount


@register.tag
def convert_amount(parser, token):
    return ConvertAmountNode.handle_token(parser, token)



from django.shortcuts import render
from django.views.decorators.http import require_POST
from django.http import HttpResponseRedirect, HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings

from . import signals
from .util import set_cookie, chunks
from .settings import CURRENCY_COOKIE_NAME, CURRENCY_LABELS, CURRENCY_CHOICES


@require_POST
@csrf_exempt
def dashboard(request, status=None):
    currency_choices = dict(CURRENCY_CHOICES)

    if 'submit' in request.POST and 'redirect_url' in request.POST:
        cookie_domain = settings.SESSION_COOKIE_DOMAIN

        if cookie_domain and cookie_domain.startswith('.'):
            cookie_domain = cookie_domain % {'host': '.'.join(request.get_host().split('.')[-2:])}

        parameters = request.POST

        response = HttpResponseRedirect(parameters.get('redirect_url'))

        keys = {
            'currency_code': {
                'choices': currency_choices,
                'cookie_name': CURRENCY_COOKIE_NAME,
                'signal': signals.currency_was_set
            },
            'language_code': {
                'choices': dict(settings.LANGUAGES),
                'cookie_name': settings.LANGUAGE_COOKIE_NAME,
                'signal': signals.language_was_set
            }
        }

        cookies = {}

        for key, value in keys.items():
            if key in parameters and parameters.get(key) in value['choices']:
                code = parameters.get(key)

                kwargs = {
                    'sender': request.__class__,
                    key: code,
                    'request': request,
                    'parameters': parameters
                }

                results = value['signal'].send(**kwargs)

                for func, result in results:
                    if result and isinstance(result, HttpResponse):
                        response = result
                        break

                cookies[value['cookie_name']] = code

        for k, v in cookies.items():
            set_cookie(response, k, v, cookie_domain=cookie_domain)

        return response

    return render(request, 'metasettings/dashboard.html', {
        'currency_labels': dict(CURRENCY_LABELS),
        'currency_choices': currency_choices,
        'currency_chunks': chunks(CURRENCY_CHOICES, 10),
        'path_info': request.POST.get('path_info', '/'),
        'status': status or 'language'
    })



from django.conf.urls import patterns, url, include
from django.http import HttpResponse


def root(request):
    return HttpResponse('Ok')


urlpatterns = patterns(
    '',
    url(r'^$',
        root,
        name='root'),

    (r'^',
        include('metasettings.urls'))
)



import datetime

from django.conf import settings


def set_cookie(response, key, value, days_expire=7, cookie_domain=None):
    if days_expire is None:
        max_age = 365 * 24 * 60 * 60  # one year
    else:
        max_age = days_expire * 24 * 60 * 60

    expires = datetime.datetime.strftime(datetime.datetime.utcnow() + datetime.timedelta(seconds=max_age), '%a, %d-%b-%Y %H:%M:%S GMT')

    response.set_cookie(key,
                        value,
                        max_age=max_age,
                        expires=expires,
                        domain=cookie_domain or settings.SESSION_COOKIE_DOMAIN,
                        secure=settings.SESSION_COOKIE_SECURE or None)

    return response


def chunks(l, n):
    for i in xrange(0, len(l), n):
        yield l[i:i + n]



from __future__ import unicode_literals

from django.db import models
from django.db.models.fields import BLANK_CHOICE_DASH
from django.utils.encoding import force_text
from django.utils.functional import lazy

from .models import Currency, currencies


class CurrencyDescriptor(object):
    def __init__(self, field):
        self.field = field

    def __get__(self, instance=None, owner=None):
        if instance is None:
            raise AttributeError(
                "The '%s' attribute can only be accessed from %s instances."
                % (self.field.name, owner.__name__))
        return Currency(
            code=instance.__dict__[self.field.name],
        )

    def __set__(self, instance, value):
        if value is not None:
            value = force_text(value)
        instance.__dict__[self.field.name] = value


class CurrencyField(models.CharField):
    descriptor_class = CurrencyDescriptor

    def __init__(self, *args, **kwargs):
        currencies_class = kwargs.pop('currencies', None)
        self.currencies = currencies_class() if currencies_class else currencies
        self.blank_label = kwargs.pop('blank_label', None)

        kwargs.update({
            'choices': self.currencies,
            'max_length': 3
        })

        return super(CurrencyField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "CharField"

    def contribute_to_class(self, cls, name):
        super(CurrencyField, self).contribute_to_class(cls, name)
        setattr(cls, self.name, self.descriptor_class(self))

    def get_prep_value(self, value):
        "Returns field's value prepared for saving into a database."
        # Convert the Currency to unicode for database insertion.
        if value is None or getattr(value, 'code', '') is None:
            return None
        return force_text(value)

    def get_prep_lookup(self, lookup_type, value):
        if hasattr(value, 'code'):
            value = value.code

        return super(CurrencyField, self).get_prep_lookup(lookup_type, value)

    def get_choices(
            self, include_blank=True, blank_choice=None, *args, **kwargs):
        if blank_choice is None:
            if self.blank_label is None:
                blank_choice = BLANK_CHOICE_DASH
            else:
                blank_choice = [('', self.blank_label)]
        return super(CurrencyField, self).get_choices(
            include_blank=include_blank, blank_choice=blank_choice, *args,
            **kwargs)

    get_choices = lazy(get_choices, list)

    def deconstruct(self):
        name, path, args, kwargs = super(CurrencyField, self).deconstruct()
        kwargs.pop('choices')
        if self.currencies is not currencies:
            # Include the countries class if it's not the default countries
            # instance.
            kwargs['currencies'] = self.currencies.__class__
        return name, path, args, kwargs


try:  # pragma: no cover
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ['^metasettings\.fields\.CurrencyField'])
except ImportError:
    pass



import os
import django

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': '',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

SITE_ID = 1
DEBUG = True

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'metasettings',
    'metasettings.tests',
]

LANGUAGE_CODE = 'en'

SECRET_KEY = 'blabla'

ROOT_URLCONF = 'metasettings.tests.urls'

if django.VERSION <= (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'

OPENEXCHANGERATES_APP_ID = os.getenv('OPENEXCHANGERATES_APP_ID')

GEOIP_PATH = os.path.join(os.path.dirname(__file__), 'data', 'GeoIP.dat')

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)
